@@ -22,8 +22,7 @@ chmod a+x $PROJECT_DIR/manage.py
 
 
 # Run syncdb/migrate/update_index
-su - vagrant -c "$PYTHON $PROJECT_DIR/manage.py syncdb --noinput && \
-                 $PYTHON $PROJECT_DIR/manage.py migrate --noinput && \
+su - vagrant -c "$PYTHON $PROJECT_DIR/manage.py migrate --noinput && \
                  $PYTHON $PROJECT_DIR/manage.py update_index"
 
 
