@@ -376,7 +376,9 @@ class ZuluSnippet(models.Model):
 
 
 class StandardIndex(Page):
-    pass
+    """ Index for the site, not allowed to be placed anywhere """
+    parent_page_types = []
+
 
 StandardIndex.content_panels = [
     FieldPanel('title', classname="full title"),
@@ -387,14 +389,22 @@ StandardIndex.content_panels = [
 class StandardChild(Page):
     pass
 
+
 class BusinessIndex(Page):
+    """ Can be placed anywhere, can only have Business children """
     subpage_types = ['tests.BusinessChild', 'tests.BusinessSubIndex']
 
+
 class BusinessSubIndex(Page):
+    """ Can be placed under BusinessIndex, and have BusinessChild children """
     subpage_types = ['tests.BusinessChild']
+    parent_page_types = ['tests.BusinessIndex']
+
 
 class BusinessChild(Page):
+    """ Can only be placed under Business indexes, no children allowed """
     subpage_types = []
+    parent_page_types = ['tests.BusinessIndex', BusinessSubIndex]
 
 
 class SearchTest(models.Model, index.Indexed):
