@@ -8,9 +8,11 @@
 ### <img src="{% gravatar_url sometemplatevariable %}">
 ### just make sure to update the "default" image path below
 
-import urllib
 import hashlib
 
+from six import b
+from six.moves.urllib.parse import urlencode
+
 from django import template
 
 register = template.Library()
@@ -30,8 +32,8 @@ class GravatarUrlNode(template.Node):
         default = "blank"
         size = int(self.size) * 2 # requested at retina size by default and scaled down at point of use with css
 
-        gravatar_url = "//www.gravatar.com/avatar/" + hashlib.md5(email.lower()).hexdigest() + "?"
-        gravatar_url += urllib.urlencode({'s': str(size), 'd': default})
+        gravatar_url = "//www.gravatar.com/avatar/" + hashlib.md5(b(email.lower())).hexdigest() + "?"
+        gravatar_url += urlencode({'s': str(size), 'd': default})
 
         return gravatar_url
 
