@@ -9,8 +9,10 @@ from django.contrib.auth.decorators import permission_required
 from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
 from django.utils import timezone
 from django.utils.translation import ugettext as _
-from django.views.decorators.http import require_GET
+from django.utils.http import is_safe_url
+from django.views.decorators.http import require_GET, require_POST
 from django.views.decorators.vary import vary_on_headers
+from django.db.models import Count
 
 from wagtail.wagtailadmin.edit_handlers import TabbedInterface, ObjectList
 from wagtail.wagtailadmin.forms import SearchForm, CopyForm
@@ -18,7 +20,6 @@ from wagtail.wagtailadmin import tasks, signals
 
 from wagtail.wagtailcore import hooks
 from wagtail.wagtailcore.models import Page, PageRevision, get_navigation_menu_items
-from wagtail.wagtailcore.signals import page_published, page_unpublished
 
 
 @permission_required('wagtailadmin.access_admin')
@@ -38,13 +39,16 @@ def index(request, parent_page_id=None):
     pages = parent_page.get_children().prefetch_related('content_type')
 
     # Get page ordering
-    ordering = request.GET.get('ordering', 'title')
-    if ordering not in ['title', '-title', 'content_type', '-content_type', 'live', '-live', 'ord']:
-        ordering = 'title'
+    ordering = request.GET.get('ordering', '-latest_revision_created_at')
+    if ordering not in ['title', '-title', 'content_type', '-content_type', 'live', '-live', 'latest_revision_created_at', '-latest_revision_created_at', 'ord']:
+        ordering = '-latest_revision_created_at'
 
     # Pagination
     if ordering != 'ord':
-        pages = pages.order_by(ordering)
+        ordering_no_minus = ordering
+        if ordering_no_minus.startswith('-'):
+            ordering_no_minus = ordering[1:]
+        pages = pages.order_by(ordering).annotate(null_position=Count(ordering_no_minus)).order_by('-null_position', ordering)
 
         p = request.GET.get('p', 1)
         paginator = Paginator(pages, 50)
@@ -68,7 +72,7 @@ def add_subpage(request, parent_page_id):
     if not parent_page.permissions_for_user(request.user).can_add_subpage():
         raise PermissionDenied
 
-    page_types = sorted(parent_page.clean_subpage_types(), key=lambda pagetype: pagetype.name.lower())
+    page_types = sorted(parent_page.allowed_subpage_types(), key=lambda pagetype: pagetype.name.lower())
 
     if len(page_types) == 1:
         # Only one page type is available - redirect straight to the create form rather than
@@ -136,7 +140,7 @@ def create(request, content_type_app_name, content_type_model_name, parent_page_
         raise Http404
 
     # page must be in the list of allowed subpage types for this parent ID
-    if content_type not in parent_page.clean_subpage_types():
+    if content_type not in parent_page.allowed_subpage_types():
         raise PermissionDenied
 
     page = page_class(owner=request.user)
@@ -181,39 +185,31 @@ def create(request, content_type_app_name, content_type_model_name, parent_page_
         form.clean = clean
 
         if form.is_valid():
-            page = form.save(commit=False)  # don't save yet, as we need treebeard to assign tree params
+            page = form.save(commit=False)
 
             is_publishing = bool(request.POST.get('action-publish')) and parent_page_perms.can_publish_subpage()
             is_submitting = bool(request.POST.get('action-submit'))
-            go_live_at = form.cleaned_data.get('go_live_at')
-            future_go_live = go_live_at and go_live_at > timezone.now()
-            approved_go_live_at = None
 
-            if is_publishing:
-                page.has_unpublished_changes = False
-                page.expired = False
-                if future_go_live:
-                    page.live = False
-                    # Set approved_go_live_at only if is publishing
-                    # and the future_go_live is actually in future
-                    approved_go_live_at = go_live_at
-                else:
-                    page.live = True
-            else:
+            # Set live to False and has_unpublished_changes to True if we are not publishing
+            if not is_publishing:
                 page.live = False
                 page.has_unpublished_changes = True
 
-            parent_page.add_child(instance=page)  # assign tree parameters - will cause page to be saved
+            # Save page
+            parent_page.add_child(instance=page)
 
-            # Pass approved_go_live_at to save_revision
-            page.save_revision(
+            # Save revision
+            revision = page.save_revision(
                 user=request.user,
                 submitted_for_moderation=is_submitting,
-                approved_go_live_at=approved_go_live_at
             )
 
+            # Publish
+            if is_publishing:
+                revision.publish()
+
+            # Notifications
             if is_publishing:
-                page_published.send(sender=page_class, instance=page)
                 messages.success(request, _("Page '{0}' published.").format(page.title))
             elif is_submitting:
                 messages.success(request, _("Page '{0}' submitted for moderation.").format(page.title))
@@ -297,55 +293,33 @@ def edit(request, page_id):
             return cleaned_data
         form.clean = clean
 
-        if form.is_valid():
+        if form.is_valid() and not page.locked:
+            page = form.save(commit=False)
+
             is_publishing = bool(request.POST.get('action-publish')) and page_perms.can_publish()
             is_submitting = bool(request.POST.get('action-submit'))
-            go_live_at = form.cleaned_data.get('go_live_at')
-            future_go_live = go_live_at and go_live_at > timezone.now()
-            approved_go_live_at = None
 
+            # Save revision
+            revision = page.save_revision(
+                user=request.user,
+                submitted_for_moderation=is_submitting,
+            )
+
+            # Publish
             if is_publishing:
-                page.has_unpublished_changes = False
-                page.expired = False
-                if future_go_live:
-                    page.live = False
-                    # Set approved_go_live_at only if publishing
-                    approved_go_live_at = go_live_at
-                else:
-                    page.live = True
-
-                # We need save the page this way to workaround a bug
-                # in django-modelcluster causing m2m fields to not
-                # be committed to the database. See github issue #192
-                form.save(commit=False)
-                page.save()
-
-                # Clear approved_go_live_at for older revisions
-                page.revisions.update(
-                    submitted_for_moderation=False,
-                    approved_go_live_at=None,
-                )
+                revision.publish()
             else:
-                # not publishing the page
+                # Set has_unpublished_changes flag
                 if page.live:
                     # To avoid overwriting the live version, we only save the page
                     # to the revisions table
-                    form.save(commit=False)
                     Page.objects.filter(id=page.id).update(has_unpublished_changes=True)
                 else:
                     page.has_unpublished_changes = True
-                    form.save(commit=False)
                     page.save()
 
-
-            page.save_revision(
-                user=request.user,
-                submitted_for_moderation=is_submitting,
-                approved_go_live_at=approved_go_live_at
-            )
-
+            # Notifications
             if is_publishing:
-                page_published.send(sender=page.__class__, instance=page)
                 messages.success(request, _("Page '{0}' published.").format(page.title))
             elif is_submitting:
                 messages.success(request, _("Page '{0}' submitted for moderation.").format(page.title))
@@ -360,7 +334,10 @@ def edit(request, page_id):
 
             return redirect('wagtailadmin_explore', page.get_parent().id)
         else:
-            messages.error(request, _("The page could not be saved due to validation errors"))
+            if page.locked:
+                messages.error(request, _("The page could not be saved as it is locked"))
+            else:
+                messages.error(request, _("The page could not be saved due to validation errors"))
 
             edit_handler = edit_handler_class(instance=page, form=form)
             errors_debug = (
@@ -391,19 +368,9 @@ def delete(request, page_id):
         raise PermissionDenied
 
     if request.POST:
-        if page.live:
-            # fetch params to pass to the page_unpublished_signal, before the
-            # deletion happens
-            specific_class = page.specific_class
-            specific_page = page.specific
-
         parent_id = page.get_parent().id
         page.delete()
 
-        # If the page is live, send the unpublished signal
-        if page.live:
-            page_unpublished.send(sender=specific_class, instance=specific_page)
-
         messages.success(request, _("Page '{0}' deleted.").format(page.title))
 
         for fn in hooks.get_hooks('after_delete_page'):
@@ -537,19 +504,12 @@ def unpublish(request, page_id):
     if not page.permissions_for_user(request.user).can_unpublish():
         raise PermissionDenied
 
-    if request.POST:
-        parent_id = page.get_parent().id
-        page.live = False
-        page.save()
-
-        # Since page is unpublished clear the approved_go_live_at of all revisions
-        page.revisions.update(approved_go_live_at=None)
-
-        page_unpublished.send(sender=page.specific_class, instance=page.specific)
+    if request.method == 'POST':
+        page.unpublish()
 
         messages.success(request, _("Page '{0}' unpublished.").format(page.title))
 
-        return redirect('wagtailadmin_explore', parent_id)
+        return redirect('wagtailadmin_explore', page.get_parent().id)
 
     return render(request, 'wagtailadmin/pages/confirm_unpublish.html', {
         'page': page,
@@ -634,8 +594,13 @@ def set_page_position(request, page_to_move_id):
         # so don't bother to catch InvalidMoveToDescendant
 
         if position_page:
-            # Move page into this position
-            page_to_move.move(position_page, pos='left')
+            # If the page has been moved to the right, insert it to the
+            # right. If left, then left.
+            old_position = list(parent_page.get_children()).index(page_to_move)
+            if int(position) < old_position:
+                page_to_move.move(position_page, pos='left')
+            elif int(position) > old_position:
+                page_to_move.move(position_page, pos='right')
         else:
             # Move page to end
             page_to_move.move(parent_page, pos='last-child')
@@ -674,7 +639,7 @@ def copy(request, page_id):
 
         # Unpublish copied pages if we need to
         if not publish_copies:
-            new_page.get_descendants(inclusive=True).update(live=False)
+            new_page.get_descendants(inclusive=True).unpublish()
 
         # Assign user of this request as the owner of all the new pages
         new_page.get_descendants(inclusive=True).update(owner=request.user)
@@ -760,9 +725,8 @@ def approve_moderation(request, revision_id):
         messages.error(request, _("The page '{0}' is not currently awaiting moderation.").format(revision.page.title))
         return redirect('wagtailadmin_home')
 
-    if request.POST:
-        revision.publish()
-        page_published.send(sender=revision.page.__class__, instance=revision.page.specific)
+    if request.method == 'POST':
+        revision.approve_moderation()
         messages.success(request, _("Page '{0}' published.").format(revision.page.title))
         tasks.send_notification.delay(revision.id, 'approved', request.user.id)
 
@@ -779,9 +743,8 @@ def reject_moderation(request, revision_id):
         messages.error(request, _("The page '{0}' is not currently awaiting moderation.").format( revision.page.title))
         return redirect('wagtailadmin_home')
 
-    if request.POST:
-        revision.submitted_for_moderation = False
-        revision.save(update_fields=['submitted_for_moderation'])
+    if request.method == 'POST':
+        revision.reject_moderation()
         messages.success(request, _("Page '{0}' rejected for publication.").format(revision.page.title))
         tasks.send_notification.delay(revision.id, 'rejected', request.user.id)
 
@@ -806,3 +769,49 @@ def preview_for_moderation(request, revision_id):
     # pass in the real user request rather than page.dummy_request(), so that request.user
     # and request.revision_id will be picked up by the wagtail user bar
     return page.serve_preview(request, page.default_preview_mode)
+
+
+@permission_required('wagtailadmin.access_admin')
+@require_POST
+def lock(request, page_id):
+    # Get the page
+    page = get_object_or_404(Page, id=page_id)
+
+    # Check permissions
+    if not page.permissions_for_user(request.user).can_lock():
+        raise PermissionDenied
+
+    # Lock the page
+    if not page.locked:
+        page.locked = True
+        page.save()
+
+    # Redirect
+    redirect_to = request.POST.get('next', None)
+    if redirect_to and is_safe_url(url=redirect_to, host=request.get_host()):
+        return redirect(redirect_to)
+    else:
+        return redirect('wagtailadmin_explore', page.get_parent().id)
+
+
+@permission_required('wagtailadmin.access_admin')
+@require_POST
+def unlock(request, page_id):
+    # Get the page
+    page = get_object_or_404(Page, id=page_id)
+
+    # Check permissions
+    if not page.permissions_for_user(request.user).can_lock():
+        raise PermissionDenied
+
+    # Unlock the page
+    if page.locked:
+        page.locked = False
+        page.save()
+
+    # Redirect
+    redirect_to = request.POST.get('next', None)
+    if redirect_to and is_safe_url(url=redirect_to, host=request.get_host()):
+        return redirect(redirect_to)
+    else:
+        return redirect('wagtailadmin_explore', page.get_parent().id)
