@@ -4,6 +4,8 @@ from django.core import urlresolvers
 from django.core.exceptions import ImproperlyConfigured
 from django.utils.html import format_html, format_html_join
 from django.utils.translation import ugettext_lazy as _
+from django.contrib.auth.models import Permission
+from django.contrib.contenttypes.models import ContentType
 
 from wagtail.wagtailcore import hooks
 from wagtail.wagtailadmin.menu import MenuItem
@@ -87,3 +89,10 @@ def editor_js():
         """,
         urlresolvers.reverse('wagtailimages_chooser')
     )
+
+
+@hooks.register('register_permissions')
+def register_permissions():
+    image_content_type = ContentType.objects.get(app_label='wagtailimages', model='image')
+    image_permissions = Permission.objects.filter(content_type = image_content_type)
+    return image_permissions
