@@ -1,17 +1,8 @@
 from django.db.models.signals import post_save, post_delete
 from django.db import models
-from django.conf import settings
 
-from wagtail.wagtailsearch.indexed import Indexed
-from wagtail.wagtailsearch.backends import get_search_backend
-
-
-def get_search_backends():
-    if hasattr(settings, 'WAGTAILSEARCH_BACKENDS'):
-        for backend in settings.WAGTAILSEARCH_BACKENDS.keys():
-            yield get_search_backend(backend)
-    else:
-        yield get_search_backend('default')
+from wagtail.wagtailsearch.index import Indexed
+from wagtail.wagtailsearch.backends import get_search_backends
 
 
 def post_save_signal_handler(instance, **kwargs):
