@@ -1,8 +1,12 @@
+from six import StringIO
+
 from django.test import TestCase
 from django.core import management
+
 from wagtail.wagtailsearch import models
-from wagtail.tests.utils import login, unittest
-from StringIO import StringIO
+from wagtail.tests.utils import WagtailTestUtils
+from wagtail.wagtailsearch.utils import normalise_query_string
+
 
 
 class TestHitCounter(TestCase):
@@ -53,6 +57,16 @@ class TestQueryStringNormalisation(TestCase):
         for query in queries:
             self.assertNotEqual(self.query, models.Query.get(query))
 
+    def test_truncation(self):
+        test_querystring = 'a' * 1000
+        result = normalise_query_string(test_querystring)
+        self.assertEqual(len(result), 255)
+
+    def test_no_truncation(self):
+        test_querystring = 'a' * 10
+        result = normalise_query_string(test_querystring)
+        self.assertEqual(len(result), 10)
+
 
 class TestQueryPopularity(TestCase):
     def test_query_popularity(self):
@@ -92,45 +106,6 @@ class TestQueryPopularity(TestCase):
         self.assertEqual(popular_queries[1], models.Query.get("popular query"))
         self.assertEqual(popular_queries[2], models.Query.get("little popular query"))
 
-    @unittest.expectedFailure # Time based popularity isn't implemented yet
-    def test_query_popularity_over_time(self):
-        today = timezone.now().date()
-        two_days_ago = today - datetime.timedelta(days=2)
-        a_week_ago = today - datetime.timedelta(days=7)
-        a_month_ago = today - datetime.timedelta(days=30)
-
-        # Add 10 hits to a query that was very popular query a month ago
-        for i in range(10):
-            models.Query.get("old popular query").add_hit(date=a_month_ago)
-
-        # Add 5 hits to a query that is was popular 2 days ago
-        for i in range(5):
-            models.Query.get("new popular query").add_hit(date=two_days_ago)
-
-        # Get most popular queries
-        popular_queries = models.Query.get_most_popular()
-
-        # Old popular query should be most popular
-        self.assertEqual(popular_queries.count(), 2)
-        self.assertEqual(popular_queries[0], models.Query.get("old popular query"))
-        self.assertEqual(popular_queries[1], models.Query.get("new popular query"))
-
-        # Get most popular queries for past week
-        past_week_popular_queries = models.Query.get_most_popular(date_since=a_week_ago)
-
-        # Only new popular query should be in this list
-        self.assertEqual(past_week_popular_queries.count(), 1)
-        self.assertEqual(past_week_popular_queries[0], models.Query.get("new popular query"))
-
-        # Old popular query gets a couple more hits!
-        for i in range(2):
-            models.Query.get("old popular query").add_hit()
-
-        # Old popular query should now be in the most popular queries
-        self.assertEqual(past_week_popular_queries.count(), 2)
-        self.assertEqual(past_week_popular_queries[0], models.Query.get("new popular query"))
-        self.assertEqual(past_week_popular_queries[1], models.Query.get("old popular query"))
-
 
 class TestGarbageCollectCommand(TestCase):
     def test_garbage_collect_command(self):
@@ -139,15 +114,18 @@ class TestGarbageCollectCommand(TestCase):
     # TODO: Test that this command is acctually doing its job
 
 
-class TestQueryChooserView(TestCase):
+class TestQueryChooserView(TestCase, WagtailTestUtils):
     def setUp(self):
-        login(self.client)
+        self.login()
 
     def get(self, params={}):
         return self.client.get('/admin/search/queries/chooser/', params)
 
-    def test_status_code(self):
-        self.assertEqual(self.get().status_code, 200)
+    def test_simple(self):
+        response = self.get()
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, 'wagtailsearch/queries/chooser/chooser.html')
+        self.assertTemplateUsed(response, 'wagtailsearch/queries/chooser/chooser.js')
 
     def test_search(self):
         response = self.get({'q': "Hello"})
